% =========================================================================
% Code for "Pandemic Recessions and Contact Tracing"
% by Leonardo Melosi and Matthias Rottner
% Journal of the European Economic Association, forthcoming
% This code generates Figure 3 (it is called within the file main_run.m)
% =========================================================================

get_setup

% define figure properties
T_plot = 150;

addpath Simulations

load pf_221; load pf_121; load pf_30_127; 

% Required Tested
pf_221.E_tot(1:T_plot,1) = pf_221.E_tt(1:T_plot,1).*pf_221.pi_bar_tt(1:T_plot,1) + pf_221.E_ttl(1:T_plot,1).*pf_221.pi_bar_ttl(1:T_plot,1);
pf_121.E_tot(1:T_plot,1) = pf_121.E_tt(1:T_plot,1).*pf_121.pi_bar_tt(1:T_plot,1);
pf_30_127.E_tot(1:T_plot,1) = pf_30_127.E_tt(1:T_plot,1).*pf_30_127.pi_bar_tt(1:T_plot,1) ;

% Probability to be tested on aggregate
pf_221.pic_G = (((1-P.pii-P.pir)*(pf_221.pic_A(1:T_plot,1)).*pf_221.A(1:T_plot,1) + (pf_221.pic_T(1:T_plot,1)).*pf_221.T(1:T_plot,1) ) ./((1-P.pii-P.pir).*pf_221.A(1:T_plot,1)+pf_221.T(1:T_plot,1)));
pf_121.pic_G = (((1-P.pii-P.pir)*(pf_121.pic_A(1:T_plot,1)).*pf_121.A(1:T_plot,1) + (pf_121.pic_T(1:T_plot,1)).*pf_121.T(1:T_plot,1) ) ./((1-P.pii-P.pir).*pf_121.A(1:T_plot,1)+pf_121.T(1:T_plot,1)));
pf_30_127.pic_G = (((1-P.pii-P.pir)*(pf_30_127.pic_A(1:T_plot,1)).*pf_30_127.A(1:T_plot,1) + (pf_30_127.pic_T(1:T_plot,1)).*pf_30_127.T(1:T_plot,1) ) ./((1-P.pii-P.pir).*pf_30_127.A(1:T_plot,1)+pf_30_127.T(1:T_plot,1)));

%%

opts.Colors     = get(groot,'defaultAxesColorOrder');
opts.saveFolder = 'Figures/';
opts.width      = 17;
opts.height     = 20;
opts.fontType   = 'Times New Roman';
opts.fontSize   = 9;
set(0,'DefaultFigureRenderer','painters')
set(0,'DefaultLineLineWidth',1.5)
set(0,'DefaultLineMarkerSize',4)

fig = figure; clf

subplot(4,2,1);hold on;grid on;box on;
plot(100*pf_121.A(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_30_127.A(1:T_plot),'-.','Color',[64 170 64]./255,'LineWidth',1.5);
plot(100*pf_221.A(1:T_plot),'r--','LineWidth',1.5);
title('Infected (Asymptomatic)');xlabel('Weeks');ylabel('% of population')

subplot(4,2,2);hold on;grid on;box on;
plot(100*pf_121.R(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_30_127.R(1:T_plot),'-.','Color',[64 170 64]./255,'LineWidth',1.5);
plot(100*pf_221.R(1:T_plot),'r--','LineWidth',1.5)
title('Recovered');xlabel('Weeks');ylabel('% of population')

subplot(4,2,3);hold on;grid on;box on;
plot(100*pf_121.D(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_30_127.D(1:T_plot),'-.','Color',[64 170 64]./255,'LineWidth',1.5);
plot(100*pf_221.D(1:T_plot),'r--','LineWidth',1.5)
title('Deaths');xlabel('Weeks');ylabel('% of population')

subplot(4,2,4);hold on;grid on;box on;
plot(100*(pf_121.Cons(1:T_plot)-P.C)/P.C,'b','LineWidth',1.5);
plot(100*(pf_30_127.Cons(1:T_plot)-P.C)/P.C,'-.','Color',[64 170 64]./255,'LineWidth',1.5);
plot(100*(pf_221.Cons(1:T_plot)-P.C)/P.C,'r--','LineWidth',1.5)
title('Aggregate Consumption');ylabel('% dev. from initial ss');xlabel('Weeks')

subplot(4,2,5);hold on;grid on;box on;
plot(100*pf_121.lambda_c(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_30_127.lambda_c(1:T_plot),'-.','Color',[64 170 64]./255,'LineWidth',1.5);
plot(100*pf_221.lambda_c(1:T_plot),'r--','LineWidth',1.5);
ylabel('%');xlabel('Weeks');title('Consumption Tax Rate')

subplot(4,2,6);hold on;grid on;box on;
plot(100*pf_121.E_tot(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_30_127.E_tot(1:T_plot),'-.','Color',[64 170 64]./255,'LineWidth',1.5);
plot(100*pf_221.E_tot(1:T_plot),'r--','LineWidth',1.5);
plot([1:T_plot],100*pf_121.Upsilon(1:T_plot),'p:','Color',[0.9290 0.6940 0.1250],'MarkerIndices',1:9:T_plot,'LineWidth',1,'MarkerSize',2);
xlabel('Weeks');ylabel('% of population');title('Total Tests')

subplot(4,2,7);hold on;grid on;box on;
plot(100*pf_121.pic_T(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_30_127.pic_T(1:T_plot),'-.','Color',[64 170 64]./255,'LineWidth',1.5);
plot(100*pf_221.pic_T(1:T_plot),'r--','LineWidth',1.5);
ylim([0 50])
ylabel('%');xlabel('Weeks');title('Test Probabability Newly Infected \pi_t^T')

subplot(4,2,8);hold on;grid on;box on;
plot(100*pf_121.pic_A(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_30_127.pic_A(1:T_plot),'-.','Color',[64 170 64]./255,'LineWidth',1.5);
plot(100*pf_221.pic_A(1:T_plot),'r--','LineWidth',1.5);
title('Test Probability Asymptomatic \pi_t^A');ylabel('%');xlabel('Weeks')

l1 = legend('Contact Tracing','Contact Tracing + Social Distancing','Comprehensive Tracing','Location','South','Orientation','horizontal');
l1.Position(2) = 0.025;l1.Position(1) = 0.09;
% scaling
fig.Units               = 'centimeters';
fig.Position(3)         = opts.width;
fig.Position(4)         = opts.height;
set(fig.Children, 'FontName', 'Times', 'FontSize',     9);

print([opts.saveFolder 'Figure3_MR_JEEA'], '-dpng', '-r600')
print([opts.saveFolder 'Figure3_MR_JEEA'], '-depsc2', '-r600')
