% =========================================================================
% Code for "Pandemic Recessions and Contact Tracing"
% by Leonardo Melosi and Matthias Rottner
% Journal of the European Economic Association, forthcoming
% This code generates Figure B.1 in the Online Appendix
% (it is called within the file main_run.m)
% =========================================================================

get_setup

% define figure properties
T_plot = 150;

addpath Simulations

load pf_821; load pf_111; load pf_121; load pf_711

% Required Tested
pf_821.E_tot(1:T_plot,1) = zeros(length(pf_111.E_tt(1:T_plot,1)),1);
pf_111.E_tot(1:T_plot,1) = pf_111.E_tt(1:T_plot,1).*pf_111.pi_bar_tt(1:T_plot,1);
pf_121.E_tot(1:T_plot,1) = pf_121.E_tt(1:T_plot,1).*pf_121.pi_bar_tt(1:T_plot,1);
pf_711.E_tot(1:T_plot,1) = pf_711.E_tt(1:T_plot,1).*pf_711.pi_bar_tt(1:T_plot,1);
% Probability to be tested on aggregate
pf_821.pic_G = (((1-P.pii-P.pir)*(pf_821.pic_A(1:T_plot,1)).*pf_821.A(1:T_plot,1) + (pf_821.pic_T(1:T_plot,1)).*pf_821.T(1:T_plot,1) ) ./((1-P.pii-P.pir).*pf_821.A(1:T_plot,1)+pf_821.T(1:T_plot,1)));
pf_111.pic_G = (((1-P.pii-P.pir)*(pf_111.pic_A(1:T_plot,1)).*pf_111.A(1:T_plot,1) + (pf_111.pic_T(1:T_plot,1)).*pf_111.T(1:T_plot,1) ) ./((1-P.pii-P.pir).*pf_111.A(1:T_plot,1)+pf_111.T(1:T_plot,1)));
pf_121.pic_G = (((1-P.pii-P.pir)*(pf_121.pic_A(1:T_plot,1)).*pf_121.A(1:T_plot,1) + (pf_121.pic_T(1:T_plot,1)).*pf_121.T(1:T_plot,1) ) ./((1-P.pii-P.pir).*pf_121.A(1:T_plot,1)+pf_121.T(1:T_plot,1)));
pf_711.pic_G = (((1-P.pii-P.pir)*(pf_711.pic_A(1:T_plot,1)).*pf_711.A(1:T_plot,1) + (pf_711.pic_T(1:T_plot,1)).*pf_711.T(1:T_plot,1) ) ./((1-P.pii-P.pir).*pf_711.A(1:T_plot,1)+pf_711.T(1:T_plot,1)));

len = 50; % Define how far in the future
for t = 1:T-len-20  % Choose horizon
    % See Section 5.1 for derivation! 
    term_sum = [];
    for j = 0:len
    tau_j = (pf_821.T(t+j,1)./pf_821.A(t+j,1));  %tau_j
        for kk = 0:j
            if j == 0
                term_prod = 1; % Product term if j = 0
            else
                term_prod = prod(1-pf_821.pic_A(t+1:t+j)-pf_821.piq_A(t+1:t+j));  % Product term j>0
            end
        end
    term_temp = tau_j*(1-P.pir-P.pii)^j*term_prod; % Equation with pi_T
    term_sum = [term_sum; term_temp];
    end

    if t ==1
        pf_821.R_0(t) = 1*sum(term_sum);  % Total equation
    else
        pf_821.R_0(t) = (1-pf_821.pic_T(t-1)-pf_821.piq_T(t-1))*sum(term_sum); % Total equation
    end
end

for t = 1:T-len-20  % Choose horizon
    % See Section 5.1 for derivation! 
    term_sum = [];
    for j = 0:len
    tau_j = (pf_111.T(t+j,1)./pf_111.A(t+j,1));  %tau_j
        for kk = 0:j
            if j == 0
                term_prod = 1; % Product term if j = 0
            else
                term_prod = prod(1-pf_111.pic_A(t+1:t+j));  % Product term j>0
            end
        end
    term_temp = tau_j*(1-P.pir-P.pii)^j*term_prod; % Equation with pi_T
    term_sum = [term_sum; term_temp];
    end

    if t ==1
        pf_111.R_0(t) = 1*sum(term_sum);  % Total equation
    else
        pf_111.R_0(t) = (1-pf_111.pic_T(t-1))*sum(term_sum); % Total equation
    end
end

for t = 1:T-len-20  % Choose horizon
    % See Section 5.1 for derivation! 
    term_sum = [];
    for j = 0:len
    tau_j = (pf_121.T(t+j,1)./pf_121.A(t+j,1));  %tau_j
        for kk = 0:j
            if j == 0
                term_prod = 1; % Product term if j = 0
            else
                term_prod = prod(1-pf_121.pic_A(t+1:t+j));  % Product term j>0
            end
        end
    term_temp = tau_j*(1-P.pir-P.pii)^j*term_prod; % Equation with pi_T
    term_sum = [term_sum; term_temp];
    end

    if t ==1
        pf_121.R_0(t) = 1*sum(term_sum);  % Total equation
    else
        pf_121.R_0(t) = (1-pf_121.pic_T(t-1))*sum(term_sum); % Total equation
    end
end

for t = 1:T-len-20  % Choose horizon
    % See Section 5.1 for derivation! 
    term_sum = [];
    for j = 0:len
    tau_j = (pf_711.T(t+j,1)./pf_711.A(t+j,1));  %tau_j
        for kk = 0:j
            if j == 0
                term_prod = 1; % Product term if j = 0
            else
                term_prod = prod(1-pf_711.pic_A(t+1:t+j)-pf_711.piq_A(t+1:t+j));  % Product term j>0
            end
        end
    term_temp = tau_j*(1-P.pir-P.pii)^j*term_prod; % Equation with pi_T
    term_sum = [term_sum; term_temp];
    end

    if t ==1
        pf_711.R_0(t) = 1*sum(term_sum);  % Total equation
    else
        pf_711.R_0(t) = (1-pf_711.pic_T(t-1)-pf_711.piq_T(t-1))*sum(term_sum); % Total equation
    end
end
%%

opts.Colors     = get(groot,'defaultAxesColorOrder');
opts.saveFolder = 'Figures/';
opts.width      = 17;
opts.height     = 20;
opts.fontType   = 'Times New Roman';
opts.fontSize   = 9;
set(0,'DefaultFigureRenderer','painters')
set(0,'DefaultLineLineWidth',1.5)
set(0,'DefaultLineMarkerSize',4)
fig = figure; clf

subplot(4,2,1);hold on;grid on;box on;
plot(100*pf_111.A(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_121.A(1:T_plot),'r--','LineWidth',1.5);
plot([1:T_plot T_plot],100*pf_711.A([1:T_plot T_plot]),'k:','LineWidth',1.5,'MarkerSize',2)
plot([1:5:T_plot T_plot],100*pf_711.A([1:5:T_plot T_plot]),'kx','LineWidth',1.5,'MarkerSize',2)
plot(100*pf_821.A(1:T_plot),'-.','Color',[64 170 64]./255,'LineWidth',1.5);
title('Infected (Asymptomatic)');ylabel('% of population');xlabel('Weeks')

subplot(4,2,2);hold on;grid on;box on;
plot(100*pf_111.R(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_121.R(1:T_plot),'r--','LineWidth',1.5);
plot([1:T_plot T_plot],100*pf_711.R([1:T_plot T_plot]),'k:','LineWidth',1.5,'MarkerSize',2);
plot([1:5:T_plot T_plot],100*pf_711.R([1:5:T_plot T_plot]),'kx','LineWidth',1.5,'MarkerSize',2);
plot(100*pf_821.R(1:T_plot),'-.','Color',[64 170 64]./255,'LineWidth',1.5)
title('Recovered');xlabel('Weeks');ylabel('% of population')

subplot(4,2,3);hold on;grid on;box on;
plot(100*pf_111.D(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_121.D(1:T_plot),'r--','LineWidth',1.5);
plot([1:T_plot T_plot],100*pf_711.D([1:T_plot T_plot]),'k:','LineWidth',1.5,'MarkerSize',2)
plot([1:5:T_plot T_plot],100*pf_711.D([1:5:T_plot T_plot]),'kx','LineWidth',1.5,'MarkerSize',2)
plot(100*pf_821.D(1:T_plot),'-.','Color',[64 170 64]./255,'LineWidth',1.5)
title('Deaths');xlabel('Weeks');ylabel('% of population')

subplot(4,2,4);hold on;grid on;box on;
plot(100*(pf_111.Cons(1:T_plot)-P.C)/P.C,'b','LineWidth',1.5);
plot(100*(pf_121.Cons(1:T_plot)-P.C)/P.C,'r--','LineWidth',1.5);
plot([1:T_plot T_plot],100*(pf_711.Cons([1:T_plot T_plot])-P.C)/P.C,'k:','LineWidth',1.5,'MarkerSize',2);
plot([1:5:T_plot T_plot],100*(pf_711.Cons([1:5:T_plot T_plot])-P.C)/P.C,'kx','LineWidth',1.5,'MarkerSize',2);
plot(100*(pf_821.Cons(1:T_plot)-P.C)/P.C,'-.','Color',[64 170 64]./255,'LineWidth',1.5)
title('Aggregate Consumption');ylabel('% dev. from initial ss');xlabel('Weeks')

subplot(4,2,5);hold on;grid on;box on;
plot(100*pf_111.E_tot(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_121.E_tot(1:T_plot),'r--','LineWidth',1.5);
plot([1:5:T_plot T_plot],0*pf_711.E_tot([1:5:T_plot T_plot]),'kx:','LineWidth',1.5,'MarkerSize',2);
plot(100*min(pf_821.E_tt(1:T_plot),pf_821.Upsilon(1:T_plot)),'-.','Color',[64 170 64]./255,'LineWidth',1.5);
ylim([0 6])
title('Total Administered Tests');xlabel('Weeks');ylabel('% of population')

subplot(4,2,6);hold on;grid on;box on;
plot(0*pf_111.E_tot(1:T_plot),'b','LineWidth',1.5);
plot(0*pf_121.E_tot(1:T_plot),'r--','LineWidth',1.5);
plot(100*pf_711.E_tot([1:T_plot T_plot]),'k:','LineWidth',1.5,'MarkerSize',2);
plot([1:5:T_plot T_plot],100*pf_711.E_tot([1:5:T_plot T_plot]),'kx','LineWidth',1.5,'MarkerSize',2);
plot(100*max(pf_821.E_tt(1:T_plot)-pf_821.Upsilon(1:T_plot),0),'-.','Color',[64 170 64]./255,'LineWidth',1.5);
ylim([0 6])
title('New Imposed Quarantines');xlabel('Weeks');ylabel('% of population')

subplot(4,2,7);hold on;grid on;box on;
plot(100*pf_111.pic_T(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_121.pic_T(1:T_plot),'r--','LineWidth',1.5);
plot([1:T_plot T_plot],100*pf_711.pic_T([1:T_plot T_plot]),'k:','LineWidth',1.5,'MarkerSize',2);
plot([1:5:T_plot T_plot],100*pf_711.pic_T([1:5:T_plot T_plot]),'kx','LineWidth',1.5,'MarkerSize',2);
plot(100*pf_821.pic_T(1:T_plot),'-.','Color',[64 170 64]./255,'LineWidth',1.5);
ylim([0 50])
title('Test Prob. Newly Infected \pi');ylabel('%');xlabel('Weeks')

subplot(4,2,8);hold on;grid on;box on;
plot(100*pf_111.piq_T(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_121.piq_T(1:T_plot),'r--','LineWidth',1.5);
plot([1:5:T_plot T_plot],100*pf_711.piq_T([1:5:T_plot T_plot]),'kx:','LineWidth',1.5,'MarkerSize',2);
plot(100*pf_821.piq_T(1:T_plot),'-.','Color',[64 170 64]./255,'LineWidth',1.5);
ylim([0 50])
ylabel('%');xlabel('Weeks');title('Quarantine Prob. Newly Infected \pi^T')


[l1] = legend({'Unlimited Tests','Limited Tests','All Exposed Quarantined','Limited Tests+ Tighter Quarantine'},'Location','South','Orientation','horizontal');
l1.Position(2) = 0.025;l1.Position(1) = -0.01;
x1 = 20; x2= 18;
l1.ItemTokenSize = [x1,x2];

% scaling
fig.Units               = 'centimeters';
fig.Position(3)         = opts.width;
fig.Position(4)         = opts.height;
set(fig.Children,'FontName','Times','FontSize',9);

print([opts.saveFolder 'FigureB1_MR_JEEA'], '-dpng', '-r600')
print([opts.saveFolder 'FigureB1_MR_JEEA'], '-depsc2', '-r600')
