% =========================================================================
% Code for "Pandemic Recessions and Contact Tracing"
% by Leonardo Melosi and Matthias Rottner
% Journal of the European Economic Association, forthcoming
% This code generates Figure I.1 in the Online Appendix
% (it is called within the file main_run.m)
% =========================================================================

get_setup

opts.Colors     = get(groot,'defaultAxesColorOrder');
opts.saveFolder = 'Figures/';
opts.width      = 17;
opts.height     = 8;
opts.fontType   = 'Times New Roman';
opts.fontSize   = 9;
set(0,'DefaultFigureRenderer','painters')
set(0,'DefaultLineLineWidth',1.5)
set(0,'DefaultLineMarkerSize',4)

fig = figure; clf
subplot(1,2,1);hold on;box on;
plot(0:0.05:P.C+150,round(P.varphi_c*(0:0.05:P.C+150)),'Color', [0 0.4470 0.7410]);
plot(P.C,round(P.varphi_c*(P.C)),'r*','MarkerSize',5)
xlabel('Consumption $c_t$','Interpreter','latex');ylabel('$\varphi_C(c_t)$','Interpreter','latex');title('Consumption Interactions')
xlim([0 P.C+150])

subplot(1,2,2);hold on;box on;
plot(0:0.05:P.N+3,round(P.varphi_n*(0:0.05:P.N+3)),'Color', [0 0.4470 0.7410])
plot(P.N,round(P.varphi_n*(P.N)),'r*','MarkerSize',5)
xlim([0 P.N+3])
xlabel('Labor $n_t$','Interpreter','latex');ylabel('$\varphi_N(n_t)$','Interpreter','latex');title('Labor Interactions')

legend('Function','Steady State','Location','SouthEast')

% scaling
fig.Units               = 'centimeters';
fig.Position(3)         = opts.width;
fig.Position(4)         = opts.height;
set(fig.Children,'FontName','Times','FontSize',9);

print([opts.saveFolder 'FigureI1_MR_JEEA'], '-dpng', '-r600')
print([opts.saveFolder 'FigureI1_MR_JEEA'], '-depsc2', '-r600')