% =========================================================================
% Code for "Pandemic Recessions and Contact Tracing"
% by Leonardo Melosi and Matthias Rottner
% Journal of the European Economic Association, forthcoming
% This function helps to solve the optimal policy for the economy without 
% tracing
% =========================================================================

function [pf_U,pf_n_s,pf_n_i,pf_n_r ] = eqm_Exposure_get_err(pf_n_s,pf_n_i,pf_n_r,ups_temp,P,T)  %%  SIRAC model



%% Create Policy Functions

% Exogenous shock - government function
pf_lambda_c = zeros(T,1);  % 
% Other policy functions 
pf_c_r = zeros(T,1); % Consumption of recovered
pf_lambdab_r =  zeros(T,1); % Marginal utility of recovered
pf_u_r = zeros(T,1); % Utility of recovered in period t
pf_Util_r = zeros(T+1,1);  % Total Utility
pf_c_i = zeros(T,1); % Consumption of infected
pf_lambdab_i =  zeros(T,1); % Marginal utility of infected
pf_u_i = zeros(T,1); % Utility of infected in period t
pf_Util_i = zeros(T+1,1);  % Total Utility
pf_c_a = zeros(T,1); % Consumption of asymptotic infected
pf_u_a = zeros(T,1); % Utility of asympotic infected in period t
pf_Util_a = zeros(T+1,1);  % Total Utility of asymptotic infected
pf_c_s = zeros(T,1); % Consumption of susceptible
pf_lambdab_s =  zeros(T,1); % Marginal utility of susceptible
pf_lambdatau_s = zeros(T,1); % Another marginal utility of susceptible
pf_u_s = zeros(T,1); % Utility of susceptible in period t
pf_Util_s = zeros(T+1,1);  % Total Utility
pf_Lambda = zeros(T,1);  % Policy Function
pf_Util_ur = zeros(T+1,1);  % Unobsered Recovered Utility

% State variables for population
pf_T   = zeros(T,1);  % Case of New Transmission in t+1
pf_tau = zeros(T,1);
pf_pid_t = zeros(T,1);
pf_Pop = ones(T,1);  % Total Population
pf_S   = ones(T,1);  % Susceptible
pf_I   = zeros(T,1);  % Infected
pf_R   = zeros(T,1);  % Recovered
pf_RA   = zeros(T,1);  % Recovered
pf_RI   = zeros(T,1);  % Recovered
pf_A   = zeros(T,1);  % Asymptotic
pf_D   = zeros(T,1);  % Deceased


pf_Cons_Meet = zeros(T,1); 
pf_N_Meet = zeros(T,1); 
pf_Pop_Meet = zeros(T,1); 
pf_pi_s1 = zeros(T,1);
pf_pi_s2 = zeros(T,1); 
pf_pi_s3 = zeros(T,1);

% Labor policy functions for the different periods
pf_n_a = zeros(T,1);%P.N*ones(T,1);  % Labor of asymptotic 



% Store updated policy functions
pf_up_n_r = zeros(T,1); 
pf_up_n_i = zeros(T,1);
pf_up_n_s = zeros(T,1);

% Calculuate 
pf_Util_r(T+1,1) = P.Util;  % This is the future
pf_Util_i(T+1,1) = P.Util_i;  %P.Util;
pf_Util_s(T+1,1) = P.Util;
pf_Util_ur(T+1,1) = P.Util;
pf_Util_a(T+1,1) = P.Util_a;



%% Main Model

pf_lambda_c(1:length(ups_temp)) = ups_temp; 

 
   dist_total = 100;
   converged = 0;  % Inititaluz
    j = 1;  % Number of Iteration
 
while converged == 0
     if dist_total < 2
        P_up_w = 0.975;
     else 
         P_up_w = 0.975;
     end


    
    % Aggregate Consumption
    pf_Cons = pf_S.*pf_c_s+       pf_I.*pf_c_i + pf_RA.*pf_c_s + pf_RI.*pf_c_r + pf_A.*pf_c_s;
      
    % Wage  
    pf_w = ones(T,1)*P.A;
    
    % Compute a sequence of remaining unknown variables using backward
    % iteration
    
    for t = T:-1:1


        % Block for the recovered
        % Marginal utility of recovered
        pf_lambdab_r(t,1) = P.theta*pf_n_r(t,1).^(1/P.eta-1)./(pf_w(t,1));
        % Consumption of recovered
        pf_c_r(t,1) = ((1+pf_lambda_c(t,1)).*pf_lambdab_r(t,1) ).^-1;
        % Utility of recovered in period t
        pf_u_r(t,1) = log(pf_c_r(t,1)) - P.theta/(1/P.eta)*(pf_n_r(t,1)).^(1/P.eta);
        % Utility function
        pf_Util_r(t,1) = pf_u_r(t,1) + P.beta.* pf_Util_r(t+1,1);
        % Lambda via Budget Constraint of Recovered
        pf_Lambda(t,1) = (1+pf_lambda_c(t,1))*pf_c_r(t,1) - pf_w(t,1)*pf_n_r(t,1);

        % Block for the infected
        % Marginal uility of infected 
        pf_lambdab_i(t,1) = P.theta/(pf_w(t,1)*P.phi).* pf_n_i(t,1).^(1/P.eta-1);
        % Consumption of infected 
        pf_c_i(t,1) = ((1+P.alpha_c*pf_lambda_c(t,1)+P.mu_cQ).*pf_lambdab_i(t,1) ).^-1;  
        % Utility of recovered in period t
        pf_u_i(t,1) = log(pf_c_i(t,1)) - P.theta/(1/P.eta)*(pf_n_i(t,1)).^(1/P.eta); 
        % Lambda Q over the budget constraint
        

    

        % Block for the susceptible
        % Consumption of susceptible (via budget constraint)
        pf_c_s(t,1) = (1/(1+pf_lambda_c(t,1)))*(pf_w(t,1)*pf_n_s(t,1) + pf_Lambda(t,1) );      
        % Utility of recovered in period t
        pf_u_s(t,1) = log(pf_c_s(t,1)) - P.theta/(1/P.eta)*(pf_n_s(t,1)).^(1/P.eta);   
        % Block for the asymptotic        

        pf_c_a(t,1) = pf_c_s(t,1);
        pf_n_a(t,1) = pf_n_s(t,1);
        pf_u_a(t,1) = pf_u_s(t,1);

    end

    % Use forward iteration:
    pf_Pop(1,1) = 1;
    pf_A(1,1) = P.eps_A*P.epsilon;
    pf_I(1,1) = P.eps_I*P.epsilon;
    pf_S(1,1) = pf_Pop(1,1)-pf_A(1,1)-pf_I(1,1);

    pf_RA(1,1) = P.eps_RA*P.epsilon;
    pf_RI(1,1) = P.eps_RI*P.epsilon;
    pf_R(1,1) = pf_RA(1,1) + pf_RI(1,1) ;
    pf_D(1,1) = 0;


    for t = 1:T

        % Death Rate
        pf_pid_t(t,1) = P.pid + P.kappa*pf_I(t,1)^2;

        %%%%%%%%%%% Calculate T %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Aggregate Consumption for Meeting 
        pf_Cons_Meet(t,1) = pf_S(t,1).*pf_c_s(t,1) + pf_RA(t,1).*pf_c_s(t,1) + pf_RI(t,1).*pf_c_r(t,1) + pf_A(t,1).*pf_c_s(t,1)  ;
        % Aggregate Labor  for Meeting
        pf_N_Meet(t,1) = pf_S(t,1).*pf_n_s(t,1) + pf_RA(t,1).*pf_n_s(t,1)  + pf_RI(t,1).*pf_n_r(t,1)  + pf_A(t,1).*pf_n_s(t,1) ;
        % Population
        pf_Pop_Meet(t,1) = pf_Pop(t,1) - pf_I(t,1);
        % Some aggregate values and for the asymptomatics / necessary for
        % newly infected
        C_A = pf_c_s(t,1)*pf_A(t,1);
        N_A = pf_n_s(t,1)*pf_A(t,1);
        C_Pop = pf_Cons_Meet(t,1); %*pf_Pop_Sir(t,1); % Note that the calculation already includes the reduced population weight
        N_Pop = pf_N_Meet(t,1); %*pf_Pop_Sir(t,1);

        

        % Law of Motion for Newly Infected(Linearized)
        k_bar = (P.varphi_c*pf_c_s(t,1))*C_A/C_Pop + (P.varphi_n*pf_n_s(t,1))*N_A/N_Pop + (P.varphi)*pf_A(t,1)/pf_Pop_Meet(t,1);
         if t >0 % == 1
         k_bar_init = k_bar;
        end
        Xi = -log(1-P.tau)*(1-P.tau)^k_bar_init; 
        T_new = Xi*k_bar*pf_S(t,1); 

        % Define pi_1,pi_2,pi_3 (Note that they are time-varying
        pf_pi_s1(t,1) = Xi*P.varphi_c;
        pf_pi_s2(t,1) = Xi*P.varphi_n; 
        pf_pi_s3(t,1) = Xi*P.varphi;

        % New cases 
        pf_T(t,1) = T_new;

        % Relative Infection Probability for susceptible               
        pf_tau(t,1) = pf_T(t,1)./pf_S(t,1);


        if t < T

        % Evolvement of Groups
        pf_S(t+1,1) =  (pf_S(t,1) - pf_T(t,1)) ;   % Susceptible  
        pf_A(t+1,1) = (1-P.pii-P.pir)*(1)*pf_A(t,1) + ...
                        (1)*pf_T(t,1) ;                
        pf_I(t+1,1) = pf_I(t,1) + P.pii*(pf_A(t,1)) - (pf_pid_t(t,1) + P.pir)*pf_I(t,1);
        pf_RA(t+1,1) = (1)*pf_RA(t,1) + P.pir*(pf_A(t,1));
        pf_RI(t+1,1) = pf_RI(t,1) +  P.pir*pf_I(t,1);
        pf_R(t+1,1) = pf_RI(t+1,1) + pf_RA(t+1,1);
        pf_D(t+1,1) = pf_D(t,1) + pf_pid_t(t,1)*pf_I(t,1);
        
        
        % Level of Population        
        pf_Pop(t+1,1) = pf_S(t+1,1) + pf_I(t+1,1) + pf_R(t+1,1) + pf_A(t+1,1); % - pf_D(t+1,1)   

        end
    end
   

    % Iterate backwards again to calculate utility
    for t = T:-1:1
       
        
        % Utility of infected function
        pf_Util_i(t,1) = pf_u_i(t,1) + P.beta.*( (1-pf_pid_t(t,1) - P.pir)*pf_Util_i(t+1,1) + P.pir*pf_Util_r(t+1,1) );
        
                                                           

        
        % Utility of susceptibles
        pf_Util_s(t,1) = pf_u_s(t,1) + P.beta.*( (1-pf_tau(t,1)).* (  (1)*pf_Util_s(t+1,1)   )...
            + pf_tau(t,1).*(  (1 )* pf_Util_a(t+1,1) ) );
            
         % Utility of asymptotic infected
         pf_Util_a(t,1) = pf_u_s(t,1) + P.beta.*( P.pii*pf_Util_i(t+1,1)  + P.pir*pf_Util_ur(t+1,1) ...
                         + (1-P.pii-P.pir)*( (1)*pf_Util_a(t+1,1) )   );
        
        % Utility of undetected recovered  
        pf_Util_ur(t,1) =  pf_u_s(t,1) + P.beta.*(pf_Util_ur(t+1,1) );
        
    end


    % Proceed forwards to calculate some remaining parameters

    for t = 1:T
        % changes with it
        pf_lambdatau_s(t,1) = P.beta*((  (1)* pf_Util_a(t+1)  ) ...
        - ( pf_Util_s(t+1,1) ) );
        pf_lambdab_s(t,1)  = (1./(1+pf_lambda_c(t,1)))*(pf_c_s(t,1).^(-1) + pf_lambdatau_s(t,1)*pf_pi_s1(t,1)*( pf_A(t,1)*pf_c_s(t,1)./pf_Cons_Meet(t,1) ) );
    end
    

    % System of Equations to be solved each period
    % Final Labor

 
  
    for t= 1:T
        start  = [pf_n_r(t,1) pf_n_i(t,1) pf_n_s(t,1)]';
        argzero = csolve('eqm_SIRAC_get_err',start,[],1e-8,20,t,P,pf_lambda_c,pf_lambdab_s,pf_lambdatau_s,pf_pi_s2, ...
                            pf_c_s,pf_c_i,pf_c_r,pf_n_s,pf_w,pf_N_Meet,pf_S,pf_I,pf_A,pf_R,pf_RI,pf_RA);
      
        pf_up_n_r(t,1) = argzero(1); 
        pf_up_n_i(t,1) = argzero(2);
        pf_up_n_s(t,1) = argzero(3);


    end


    pf_n_r = P_up_w*pf_n_r + (1-P_up_w)*pf_up_n_r;
    pf_n_i = P_up_w*pf_n_i + (1-P_up_w)*pf_up_n_i;
    pf_n_s = P_up_w*pf_n_s + (1-P_up_w)*pf_up_n_s;
  
    
    dist_nr = max(abs(pf_n_r(:) -pf_up_n_r(:)));
    dist_ni = max(abs(pf_n_i(:) -pf_up_n_i(:)));
    dist_ns = max(abs(pf_n_s(:) -pf_up_n_s(:)));




    dist_total = max([dist_nr dist_ni dist_ns]);

    if dist_total < P.tol 
        converged = 1;
    end

if rem(j,10) == 0
   % disp(['Iteration: ' num2str(j) ' Distance: ' num2str(dist_total)])
end
  %  
    if dist_total < P.tol 
        converged = 1;
     %     disp(['Iteration: ' num2str(j) ' Distance: ' num2str(dist_total)])
    end


 
    j = j+1;
    
    if j > 500 || dist_total > 100000
    
        error('Too long for convergence')
    end
   
  
    
end


% % Utility
pf_U = pf_S.*pf_Util_s(1:end-1) + pf_I.*pf_Util_i(1:end-1) + pf_RA.*pf_Util_r(1:end-1) + pf_RI.*pf_Util_r(1:end-1) + pf_A.*pf_Util_a(1:end-1) ;



