% =========================================================================
% Code for "Pandemic Recessions and Contact Tracing"
% by Leonardo Melosi and Matthias Rottner
% Journal of the European Economic Association, forthcoming
% This function helps to solve the optimal policy for the economy with 
% contact tracing
% =========================================================================


function [pf_U,pf_n_s,pf_n_i,pf_n_r, pf_n_c ] = eqm_Exposure_trace_get_err(pf_n_s,pf_n_i,pf_n_r,pf_n_c,ups_temp,P,T)  %%  SIRAC model



%% Create Policy Functions

% Exogenous shock - government function
pf_lambda_c = zeros(T,1);  % 
% Other policy functions 
pf_c_r = zeros(T,1); % Consumption of recovered
pf_lambdab_r =  zeros(T,1); % Marginal utility of recovered
pf_u_r = zeros(T,1); % Utility of recovered in period t
pf_Util_r = zeros(T+1,1);  % Total Utility
pf_c_i = zeros(T,1); % Consumption of infected
pf_lambdab_i =  zeros(T,1); % Marginal utility of infected
pf_u_i = zeros(T,1); % Utility of infected in period t
pf_Util_i = zeros(T+1,1);  % Total Utility
pf_c_c = zeros(T,1); % Consumption of checked infected
pf_lambdab_c =  zeros(T,1); % Marginal utility of checked infected 
pf_u_c = zeros(T,1); % Utility of checked infected in period t
pf_Util_c = zeros(T+1,1);  % Total Utility of asymptotic infected
pf_c_a = zeros(T,1); % Consumption of asymptotic infected
pf_u_a = zeros(T,1); % Utility of asympotic infected in period t
pf_Util_a = zeros(T+1,1);  % Total Utility of asymptotic infected
pf_c_s = zeros(T,1); % Consumption of susceptible
pf_lambdab_s =  zeros(T,1); % Marginal utility of susceptible
pf_lambdatau_s = zeros(T,1); % Another marginal utility of susceptible
pf_u_s = zeros(T,1); % Utility of susceptible in period t
pf_Util_s = zeros(T+1,1);  % Total Utility
pf_Lambda = zeros(T,1);  % Policy Function
pf_Util_ur = zeros(T+1,1);  % Unobsered Recovered Utility

% State variables for population
pf_T   = zeros(T,1);  % Case of New Transmission in t+1
pf_tau = zeros(T,1);
pf_tau_exact = zeros(T,1);
pf_pid_t = zeros(T,1);
pf_Pop = ones(T,1);  % Total Population
pf_S   = ones(T,1);  % Susceptible
pf_I   = zeros(T,1);  % Infected
pf_R   = zeros(T,1);  % Recovered
pf_RA   = zeros(T,1);  % Recovered
pf_RI   = zeros(T,1);  % Recovered
pf_C   = zeros(T,1);
pf_A   = zeros(T,1);  % Asymptotic
pf_D   = zeros(T,1);  % Deceased


pf_Cons_Meet = zeros(T,1); 
pf_N_Meet = zeros(T,1); 
pf_Pop_Meet = zeros(T,1); 
pf_pi_s1 = zeros(T,1);
pf_pi_s2 = zeros(T,1); 
pf_pi_s3 = zeros(T,1);

% Labor policy functions for the different periods
pf_n_a = zeros(T,1);%P.N*ones(T,1);  % Labor of asymptotic 


% Probability of being traced
pf_pic = zeros(T,1);
pf_Upsilon = zeros(T,1); % Amount of Tests
pf_pic_A = zeros(T,1); % Probability of getting tested
pf_pic_T = zeros(T,1); % Probability of getting tested
pf_fk        = zeros(T,length(P.grid));

% Distribution of contacts
pf_g_A = zeros(P.k_s_max+1,T);
pf_g_ST = zeros(P.k_s_max+1,T);
pf_g_RA = zeros(P.k_s_max+1,T);


% Store updated policy functions
pf_up_n_r = zeros(T,1); 
pf_up_n_i = zeros(T,1);
pf_up_n_s = zeros(T,1);
pf_up_n_c = zeros(T,1);

% Calculuate 
pf_Util_r(T+1,1) = P.Util;  % This is the future
pf_Util_i(T+1,1) = P.Util_i;  %P.Util;
pf_Util_s(T+1,1) = P.Util;
pf_Util_ur(T+1,1) = P.Util;
pf_Util_a(T+1,1) = P.Util_a;
pf_Util_c(T+1,1) = P.Util_c;


%% Main Model

pf_lambda_c(1:length(ups_temp)) = ups_temp; 


if P.Choice_Ups == 1  %Flatrate of 10% Tests
    pf_Upsilon(1:T) = P.Ups_flatrate; 
elseif P.Choice_Ups == 2  % Limited Tests
    pf_Upsilon(1:29) = linspace(P.Ups_start_w11,P.Ups_end_w40,29);  
    P.step_s = pf_Upsilon(2)-pf_Upsilon(1);
    pf_Upsilon(30:52) = P.Ups_end_w40+P.step_s :P.step_s:P.Ups_end_w40 +23*P.step_s;
    pf_Upsilon(53:T) =   pf_Upsilon(52);
end

   dist_total = 100;
   converged = 0;  % Inititaluz
    j = 1;  % Number of Iteration
 
while converged == 0
     if dist_total < 2
        P_up_w = 0.975;
     else 
         P_up_w = 0.975;
     end


    
    % Aggregate Labor and Consumption
    pf_Cons = pf_S.*pf_c_s+       pf_I.*pf_c_i + pf_RA.*pf_c_s + pf_RI.*pf_c_r + pf_A.*pf_c_s;
      
    % Wage  
    pf_w = ones(T,1)*P.A;
    
    % Compute a sequence of remaining unknown variables using backward
    % iteration
    
    for t = T:-1:1


        % Block for the recovered
        % Marginal utility of recovered
        pf_lambdab_r(t,1) = P.theta*pf_n_r(t,1).^(1/P.eta-1)./(pf_w(t,1));
        % Consumption of recovered
        pf_c_r(t,1) = ((1+pf_lambda_c(t,1)).*pf_lambdab_r(t,1) ).^-1;
        % Utility of recovered in period t
        pf_u_r(t,1) = log(pf_c_r(t,1)) - P.theta/(1/P.eta)*(pf_n_r(t,1)).^(1/P.eta);
        % Utility function
        pf_Util_r(t,1) = pf_u_r(t,1) + P.beta.* pf_Util_r(t+1,1);
        % Lambda via Budget Constraint of Recovered
        pf_Lambda(t,1) = (1+pf_lambda_c(t,1))*pf_c_r(t,1) - pf_w(t,1)*pf_n_r(t,1);

        % Block for the infected
        % Marginal uility of infected 
        pf_lambdab_i(t,1) = P.theta/(pf_w(t,1)*P.phi).* pf_n_i(t,1).^(1/P.eta-1);
        % Consumption of infected 
        pf_c_i(t,1) = ((1+P.alpha_c*pf_lambda_c(t,1)+P.mu_cQ).*pf_lambdab_i(t,1) ).^-1;  
        % Utility of recovered in period t
        pf_u_i(t,1) = log(pf_c_i(t,1)) - P.theta/(1/P.eta)*(pf_n_i(t,1)).^(1/P.eta); 
        % Lambda Q over the budget constraint
        
        % Block for the checked
        % Marginal uility of checked
        pf_lambdab_c(t,1) = P.theta/(pf_w(t,1)*P.phi_c).* pf_n_c(t,1).^(1/P.eta-1);
        % Consumption of checked
        pf_c_c(t,1) = ((1+P.alpha_c*pf_lambda_c(t,1)+P.mu_cQ).*pf_lambdab_c(t,1) ).^-1;  
        % Utility of checked in period t
        pf_u_c(t,1) = log(pf_c_c(t,1)) - P.theta/(1/P.eta)*(pf_n_c(t,1)).^(1/P.eta); 


    

        % Block for the susceptible
        % Consumption of susceptible (via budget constraint)
        pf_c_s(t,1) = (1/(1+pf_lambda_c(t,1)))*(pf_w(t,1)*pf_n_s(t,1) + pf_Lambda(t,1) );      
        % Utility of recovered in period t
        pf_u_s(t,1) = log(pf_c_s(t,1)) - P.theta/(1/P.eta)*(pf_n_s(t,1)).^(1/P.eta);   
        % Block for the asymptotic        

        pf_c_a(t,1) = pf_c_s(t,1);
        pf_n_a(t,1) = pf_n_s(t,1);
        pf_u_a(t,1) = pf_u_s(t,1);

    end

    % Use forward iteration:
    pf_Pop(1,1) = 1;
    pf_A(1,1) = P.eps_A*P.epsilon;
    pf_I(1,1) = P.eps_I*P.epsilon;
    pf_S(1,1) = pf_Pop(1,1)-pf_A(1,1)-pf_I(1,1);

    pf_RA(1,1) = P.eps_RA*P.epsilon;
    pf_RI(1,1) = P.eps_RI*P.epsilon;
    pf_R(1,1) = pf_RA(1,1) + pf_RI(1,1) ;
    pf_D(1,1) = 0;
    pf_C(1,1) = 0;


    for t = 1:T

        % Death Rate
        pf_pid_t(t,1) = P.pid + P.kappa*pf_I(t,1)^2;

        %%%%%%%%%%% Calculate T %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Aggregate Consumption for Meeting 
        pf_Cons_Meet(t,1) = pf_S(t,1).*pf_c_s(t,1) + pf_RA(t,1).*pf_c_s(t,1) + pf_RI(t,1).*pf_c_r(t,1) + pf_A(t,1).*pf_c_s(t,1)  ;
        % Aggregate Labor  for Meeting
        pf_N_Meet(t,1) = pf_S(t,1).*pf_n_s(t,1) + pf_RA(t,1).*pf_n_s(t,1)  + pf_RI(t,1).*pf_n_r(t,1)  + pf_A(t,1).*pf_n_s(t,1) ;
        % Population 
        pf_Pop_Meet(t,1) = pf_Pop(t,1) - pf_C(t,1) - pf_I(t,1);
        % Some aggregate values and for the asymptomatics / necessary for
        % newly infected
        C_A = pf_c_s(t,1)*pf_A(t,1);
        N_A = pf_n_s(t,1)*pf_A(t,1);
        C_Pop = pf_Cons_Meet(t,1); 
        N_Pop = pf_N_Meet(t,1); %

        if P.trace == 1  % && P.trace <10 
        % Probability mass function of contacts from susceptibles with asymptomatic 
        [f_k] = pmf_contacts_withA_get_err(pf_c_s(t,1),pf_n_s(t,1),P,pf_c_s(t,1),pf_A(t,1),pf_n_s(t,1),pf_Cons_Meet(t,1),pf_N_Meet(t,1),pf_Pop_Meet(t,1));
        pf_fk(t,:) = f_k;
        % Probability to get infected (based on Bernoulli trials)
        tau_tilde = (1 - (1-P.tau).^P.grid)';
        % Distribution of newly infected 
        pf_tau_exact(t,1) = sum(f_k.*tau_tilde);
        end

        % Law of Motion for Newly Infected(Linearized)
        k_bar = (P.varphi_c*pf_c_s(t,1))*C_A/C_Pop + (P.varphi_n*pf_n_s(t,1))*N_A/N_Pop + (P.varphi)*pf_A(t,1)/pf_Pop_Meet(t,1);
        Xi = -log(1-P.tau)*(1-P.tau)^k_bar; %
        T_new = Xi*k_bar*pf_S(t,1); %

        % Define pi_1,pi_2,pi_3 (Note that they are time-varying
        pf_pi_s1(t,1) = Xi*P.varphi_c;
        pf_pi_s2(t,1) = Xi*P.varphi_n; 
        pf_pi_s3(t,1) = Xi*P.varphi;

        % New cases 
        pf_T(t,1) = T_new;

        % Relative Infection Probability for susceptible               
        pf_tau(t,1) = pf_T(t,1)./pf_S(t,1);


        if t < T
        
            % Probability of being caught expressed as Bernoulli trial
            mu_tilde = (1 - (1-P.pii).^P.grid)';
            
            %%%% Susceptible %%%%
            % Probability of a susceptible have k contacts conditional on
            % the subject to become infected
            f_SA_T = f_k.*tau_tilde/pf_tau_exact(t,1);  % Newly Infected
            %f_SA_ST = f_k.*(1-tau_tilde)/(1-pf_tau_exact(t,1)); % Remaining Susceptibles

            % Average Probability of being Traced            
            pi_C_S_tt = f_k'*mu_tilde;  % Susceptibles at beginning of period, Asymptomatics and Untested Recovered
            pi_C_T_tt = f_SA_T'*mu_tilde; % Newly Infected
            pi_C_A_tt = pi_C_S_tt;    % Asymptomatic 
            pi_C_RA_tt = pi_C_S_tt;   % Undetected Recovered
            
           % Contacts
            pf_E_tt(t,1) = pi_C_S_tt*pf_S(t,1) + pi_C_A_tt*(1-P.pii)*pf_A(t,1) + pi_C_RA_tt*pf_RA(t,1);
            pf_pi_bar_tt(t,1) = min(pf_Upsilon(t,1)./pf_E_tt(t,1),1);
            
            % Aggregate 
            pf_pic_A(t,1)  = pi_C_A_tt*pf_pi_bar_tt(t,1)*(1-P.pi_F);
            pf_pic_T(t,1)  = pi_C_T_tt*pf_pi_bar_tt(t,1)*(1-P.pi_F);
            



        % Evolvement of Groups
        pf_S(t+1,1) =  (pf_S(t,1) - pf_T(t,1)) ;   % Susceptible  
        pf_A(t+1,1) = (1-P.pii-P.pir)*(1-pf_pic_A(t,1))*pf_A(t,1) + ...
                        (1-pf_pic_T(t,1))*pf_T(t,1) ;
        pf_C(t+1,1) = (1-P.pii-P.pir)*pf_C(t,1) + (1-P.pii-P.pir)*pf_pic_A(t,1)*pf_A(t,1) ...
                       + pf_pic_T(t,1)*pf_T(t,1);  
        pf_I(t+1,1) = pf_I(t,1) + P.pii*(pf_A(t,1)+ pf_C(t,1)) - (pf_pid_t(t,1) + P.pir)*pf_I(t,1);
        pf_RA(t+1,1) = (1)*pf_RA(t,1) + P.pir*(pf_A(t,1));
        pf_RI(t+1,1) = pf_RI(t,1) + P.pir*(pf_C(t,1)) +  P.pir*pf_I(t,1);
        pf_R(t+1,1) = pf_RI(t+1,1) + pf_RA(t+1,1);
        pf_D(t+1,1) = pf_D(t,1) + pf_pid_t(t,1)*pf_I(t,1);
        % Level of Population        
        pf_Pop(t+1,1) = pf_S(t+1,1) + pf_I(t+1,1) + pf_R(t+1,1) + pf_C(t+1,1) + pf_A(t+1,1); % 

 
   
        end
    end
   

    % Iterate backwards again to calculate utility
    for t = T:-1:1
       
        
        % Utility of infected function
        pf_Util_i(t,1) = pf_u_i(t,1) + P.beta.*( (1-pf_pid_t(t,1) - P.pir)*pf_Util_i(t+1,1) + P.pir*pf_Util_r(t+1,1) );

        % Utility of checked infected
        pf_Util_c(t,1) = pf_u_c(t,1) + P.beta.*( P.pii*pf_Util_i(t+1,1)  + P.pir*pf_Util_r(t+1,1) ...
                                                  + (1-P.pii-P.pir)*pf_Util_c(t+1,1)          ) ;                                                          

        % Utility of susceptibles
        pf_Util_s(t,1) = pf_u_s(t,1) + P.beta.*( (1-pf_tau(t,1)).* (  (1)*pf_Util_s(t+1,1)   )...
            + pf_tau(t,1).*(  (1-pf_pic_T(t,1) )* pf_Util_a(t+1,1)  + pf_pic_T(t,1)*pf_Util_c(t+1,1) ) );
            
         % Utility of asymptotic infected
         pf_Util_a(t,1) = pf_u_s(t,1) + P.beta.*( P.pii*pf_Util_i(t+1,1)  + P.pir*pf_Util_ur(t+1,1) ...
                         + (1-P.pii-P.pir)*( pf_pic_A(t,1)*pf_Util_c(t+1,1) + (1-pf_pic_A(t,1))*pf_Util_a(t+1,1) )   );
        
        % Utility of undetected recovered  
        pf_Util_ur(t,1) =  pf_u_s(t,1) + P.beta.*(pf_Util_ur(t+1,1) );
        
    end


    % Proceed forwards to calculate some remaining parameters

    for t = 1:T
        % changes with it
        pf_lambdatau_s(t,1) = P.beta*((  (1-pf_pic_T(t,1))* pf_Util_a(t+1)  + pf_pic_T(t,1)*pf_Util_c(t+1)  ) ...
        - ( pf_Util_s(t+1,1) ) );
        pf_lambdab_s(t,1)  = (1./(1+pf_lambda_c(t,1)))*(pf_c_s(t,1).^(-1) + pf_lambdatau_s(t,1)*pf_pi_s1(t,1)*( pf_A(t,1)*pf_c_s(t,1)./pf_Cons_Meet(t,1) ) );
    end
    

    % System of Equations to be solved each period
    % Final Labor

 
  
    for t= 1:T

        start  = [pf_n_r(t,1) pf_n_i(t,1) pf_n_s(t,1) pf_n_c(t,1)]';
        argzero = csolve('eqm_SIRAC_trace_get_err',start,[],1e-8,20,t,P,pf_lambda_c,pf_lambdab_s,pf_lambdatau_s,pf_pi_s2, ...
                            pf_c_s,pf_c_i,pf_c_r,pf_n_s,pf_w,pf_N_Meet,pf_S,pf_I,pf_A,pf_R,pf_RI,pf_RA,pf_c_c,pf_C);
       
        pf_up_n_r(t,1) = argzero(1); 
        pf_up_n_i(t,1) = argzero(2);
        pf_up_n_s(t,1) = argzero(3);
        pf_up_n_c(t,1) = argzero(4);

    end


    pf_n_r = P_up_w*pf_n_r + (1-P_up_w)*pf_up_n_r;
    pf_n_i = P_up_w*pf_n_i + (1-P_up_w)*pf_up_n_i;
    pf_n_s = P_up_w*pf_n_s + (1-P_up_w)*pf_up_n_s;
    pf_n_c = P.up_w*pf_n_c + (1-P.up_w)*pf_up_n_c;
    
    dist_nr = max(abs(pf_n_r(:) -pf_up_n_r(:)));
    dist_ni = max(abs(pf_n_i(:) -pf_up_n_i(:)));
    dist_ns = max(abs(pf_n_s(:) -pf_up_n_s(:)));
    dist_nc = max(abs(pf_n_c(:) -pf_up_n_c(:)));



    dist_total = max([dist_nr dist_ni dist_ns  dist_nc]);

    if dist_total < P.tol 
        converged = 1;
    end


   
    if dist_total < P.tol 
        converged = 1;
     %     disp(['Iteration: ' num2str(j) ' Distance: ' num2str(dist_total)])
    end


 
    j = j+1;
    
    if j > 500 || dist_total > 100000
        error('Too long for convergence')
    end
    
   
    
end


% % Utility
pf_U = pf_S.*pf_Util_s(1:end-1) + pf_I.*pf_Util_i(1:end-1) + pf_RA.*pf_Util_r(1:end-1) + pf_RI.*pf_Util_r(1:end-1) + pf_A.*pf_Util_a(1:end-1) + pf_C.*pf_Util_c(1:end-1) ;



